#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Archive;

use strict;
use warnings;

use APPLIB::Topic;
use APPLIB::Logger;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	$this->{SYSTEMDIR} = $confobj->SystemDir();		# システムディレクトリ
	$this->{DATADIR} = $confobj->DataDir();			# データディレクトリ
	$this->{NEWNUM} = $confobj->NewTopicNum();	
	$this->{OPENNUM} = $confobj->OpenTopicNum();	# 開いた状態で表示するトピック数
	$this->{ANONYMOUS} = $confobj->Anonymous();		# 匿名の場合の代替文字列

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	bless($this);
	return $this;
}

# 追加
sub add {
	my $this = shift;
	my $topicid = shift;

	# 既存の最近のトピックファイルを読み込み配列に保存する
	my $newtopicfile = "$this->{DATADIR}/newtopic.data";
	my $newnum = $this->{NEWNUM};
	my @newdata = ();
	my $data;

	if (-f $newtopicfile) {	# ファイルが存在する場合
		open IN, $newtopicfile;
		while ($data = <IN>) {
			chop $data;
			if ($data ne '' && $data ne $topicid) {
				push @newdata, $data;
			}
		}
		close IN;
	}

	open(OUT,">$newtopicfile");
	flock(OUT, $this->{LOCK_EX});
	seek(OUT, 0, 2);

	# 新規作成のデータを先頭に追加
	unshift(@newdata, $topicid);
	for (my $i = 0; $i < $newnum; $i++) {
		$data = shift @newdata;
		print OUT "$data\n";
	}

	flock(OUT, $this->{LOCK_UN});
	close(OUT);
	# ファイルのパーミッションを変更
	chmod 0766, $newtopicfile;
}

# 削除
sub delete {
	my $this = shift;
	my $topicid = shift;
	my $newtopicfile = "$this->{DATADIR}/newtopic.data";
	my $data;

	# 既存の最近のトピックファイルを読み込み配列に保存する
	my @newdata = ();
	open IN, $newtopicfile;
	while ($data = <IN>) {
		chop $data;
		if ($data ne "") {
			my $newtopicid = $data;
			if (!($newtopicid eq $topicid)) {
				push @newdata, $data;
			}
		}
	}
	close IN;

	# 最近のトピックファイルを更新
	open(OUT,">$newtopicfile");
	flock(OUT, $this->{LOCK_EX});
	seek(OUT, 0, 2);
	foreach $data (@newdata) {
		print OUT "$data\n";
	}
	flock(OUT, $this->{LOCK_UN});
	close(OUT);
}

# 最新トピックまたは指定されたトピックの一覧を作成する
sub topictable {
	my $this = shift;
	my @topicdata;
	my $data;
	my $newtopicfile;
	my $datafile;

	if (@_) {
		@topicdata = @_;
	}
	else {
		# 最近のトピックデータファイルを読み込みトピック一覧を作成する
		$newtopicfile = "$this->{DATADIR}/newtopic.data";
		if (-f $newtopicfile) {	# ファイルが存在する場合
			open IN, $newtopicfile;
			while (<IN>) {
				$data = $_;	# 1行読み込み
				chop($data);	# 改行を取り除く
				if ($data ne "") {
					push(@topicdata, $data);
				}
			}
			close IN;
		}
	}

	my $table = '';
	my $cnt = 0;

	my $topicobj = APPLIB::Topic->new();
	foreach my $keytopicid (@topicdata) {
		$topicobj->load($keytopicid);
		my $open = 0;
		if ($cnt < $this->{OPENNUM}) {
			$open = 1;
		}
		$table .= $topicobj->genoutline($open);
		$cnt++;
	}

	return $table;
}

# アーカイブ表の取得(HTML)
sub archivetable {
	my $this = shift;
	my $dir = "$this->{DATADIR}/*ar.data";
	my @files = <${dir}>;
	@files = reverse (sort @files);

	my $table = <<"End_of_Temp";
<h5>アーカイブ</h5>
End_of_Temp

	my $datadir = $this->{DATADIR};
	foreach my $filename (@files) {
		$filename =~ s/$datadir\///g;
		my $yyyy = substr($filename, 0, 4);
		my $mm = substr($filename, 4, 2);
		$table .= <<"End_of_Temp";
<a href="view.cgi?year=$yyyy&month=$mm">$yyyy/$mm</a><br>
End_of_Temp
	}

	return $table;
}

# トピックをアーカイブファイルに追加する
sub addarchive {
	my $this = shift;
	my $topicid = shift;
	my $lastdt = shift;
	my $yyyy = substr($lastdt, 0, 4);	# 年
	my $mm = substr($lastdt, 5, 2);		# 月

	# アーカイブデータファイルに追加
	my $archivefile = "$this->{DATADIR}/$yyyy$mm" . "ar.data";
	open(ARCHIVEFILE,">>$archivefile");
	flock(ARCHIVEFILE, $this->{LOCK_EX});
	seek(ARCHIVEFILE, 0, 2);
	print ARCHIVEFILE "$topicid,$lastdt\n";
	flock(ARCHIVEFILE, $this->{LOCK_UN});
	close(ARCHIVEFILE);
	chmod 0766, $archivefile;
}

# トピックをアーカイブファイルから削除する
sub deletearchive {
	my $this = shift;
	my $topicid = shift;
	my $lastdt = shift;
	my $yyyy = substr($lastdt, 0, 4);	# 年
	my $mm = substr($lastdt, 5, 2);		# 月
	my @olddata;
	my $atopicid;

	my $archivefile = "$this->{DATADIR}/$yyyy$mm" . "ar.data";
	open IN, $archivefile;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne "") {
			$atopicid = $data;
			if (!($atopicid eq $topicid)) {
				push @olddata, $data;
			}
		}
	}
	close IN;

	# アーカイブファイルを更新
	open(ARCHIVEFILE,">$archivefile");
	flock(ARCHIVEFILE, $this->{LOCK_EX});
	seek(ARCHIVEFILE, 0, 2);
	foreach my $data (@olddata) {
		print ARCHIVEFILE "$data\n";
	}
	flock(ARCHIVEFILE, $this->{LOCK_UN});
	close(ARCHIVEFILE);
}

# 現在のトピックカウンター値を取得する
sub lastseq {
	my $this = shift;
	my $counter = 0;

	my $counterfile = "$this->{SYSTEMDIR}/topiccounter.txt";
	if (-f $counterfile) {	# ファイルが存在する場合
		open IN, $counterfile;
		my $data = <IN>;	# 1行読み込み
		chop($data);	# 改行を取り除く
		if ($data ne '') {
			$counter = $data;
		}
		close IN;
	}

	return $counter;
}

# 最新トピックの一覧を作成する
sub newtopic {
	my $this = shift;
	my @topiclist = ();

	# 最新トピックデータファイルを読み込みトピック一覧を作成する
	my $newtopicfile = "$this->{DATADIR}/newtopic.data";
	if (-f $newtopicfile) {	# ファイルが存在する場合
		open NEWFILE, $newtopicfile;
		while (my $data = <NEWFILE>) {
			chop($data);	# 改行を取り除く
			if ($data ne '') {
				push(@topiclist, $data);
			}
		}
		close NEWFILE;
	}

	my $list = '';
	my $cnt = 0;

	my $topicobj = APPLIB::Topic->new();
	foreach my $keytopicid (@topiclist) {
		$topicobj->load($keytopicid);
		my $title = $topicobj->title();
		$list .= <<"End_of_Temp";
<a href="detail$keytopicid.html">$title</a><br>
End_of_Temp
		$cnt++;
		last if ($cnt >= 10);
	}

	if ($list ne '') {
		$list = "<h5>更新情報</h5>\n" . $list;
	}

	return $list;
}

# トピックカウンター値を更新する
sub updatecounter {
	my $this = shift;
	my $counter = shift;

	my $counterfile = "$this->{SYSTEMDIR}/topiccounter.txt";
	if (-f $counterfile) {	# ファイルが存在する場合
		# トピックカウンターファイルを更新
		open(COUNTERFILE,">$counterfile");
		flock(COUNTERFILE, $this->{LOCK_EX});
		seek(COUNTERFILE, 0, 2);
		print COUNTERFILE "$counter\n";
		flock(COUNTERFILE, $this->{LOCK_UN});
		close(COUNTERFILE);

		return 1;
	}

	return 0;
}

1;
# End of Archive.pm
