#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::String;

use strict;
use warnings;

use Jcode;

use APPLIB::Logger;
use APPLIB::Conf;

my @denylist = ();

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ

	$this->{TAIL} = '...';		# 尻尾

	bless($this);
	return $this;
}

# 尻尾
sub tail {
	my $this = shift;
	if (@_) {
		$this->{TAIL} = shift;
	}
	return $this->{TAIL};
}

# 文字列を一定の長さで切る(文字コードはシフトＪＩＳを前提)
sub cut {
	my $this = shift;
	my $srcstr = shift;	# 第一引数：文字列
	my $length = shift;	# 第二引数：切る長さ

	$srcstr =~ s/<[^>]*>//g;	# タグを取り除く
	$srcstr =~ s/\r?\n//g;		# 改行を取り除く

	# シフトJISに変換
	$srcstr = $this->encodeex('utf8', 'sjis', $srcstr);

	# 文字列が指定の長さと同じか短い場合はそのまま返す
	my $srclength = length($srcstr);
	my $deststr = '';
	if ($srclength <= $length) {
		$deststr = $srcstr;
	}
	else {
		# 指定された長さで文字列を切る
		for (my $i = 0; $i < $length; $i++) {
			my $ch = substr($srcstr, $i, 1);
			$deststr .= $ch;
			if ($ch =~ /[\x81-\x9F\xE0-\xEF]/) {
				$i++;
				$ch = substr($srcstr, $i, 1);
				$deststr .= $ch;
			}
		}
		# 文章を切った場合は末尾を付ける
		$deststr .= $this->{TAIL};
	}

	$deststr = $this->encodeex('sjis', 'utf8', $deststr);

	return $deststr;
}

# フィルターに登録されていないかチェック
sub denycheck {
	my $this = shift;
	my $checkstr = shift;
	my $denystr;
	my $result = 1;

	my $loggerobj = APPLIB::Logger->new();

	if (@denylist < 1) { # 拒否リストが読み込まれていないとき読み込む
		my $filename = "$this->{SYSTEMDIR}/filter.txt";
		if (-f $filename) {	# ファイルが存在する場合
			open IN, $filename;
			while (<IN>) {
				$denystr = $_;	# 1行読み込み
				chop($denystr);	# 改行を取り除く
				if ($denystr ne '') {
					push @denylist, $denystr;
				}
			}
			close IN;
		}
	}

	foreach $denystr (@denylist) {
		#$loggerobj->write("denycheck - denystr:$denystr");
		#$loggerobj->write("denycheck - checkstr:$checkstr");
		if ($checkstr =~ /$denystr/) {
			$result =0;
			last;
		}
	}

	return $result;
}

# HTMLタグを無効化
sub taginvalidate {
	my $this = shift;
	my $text = shift;

	$text =~ s/</&lt;/g;
	$text =~ s/>/&gt;/g;

	return $text;
}

# 文字エンコードの取得(デフォルト UTF-8)
sub getencode {
	my $this = shift;
	my $text = shift;
	my $code = 'utf8';
	my $nmatch = '';
	# 文字コードの種類を取得
	($code, $nmatch) = Jcode::getcode(\$text); 
	return $code;
}

# 文字エンコードの変換(変換先エンコード, 変換文字列) 変換した文字列を返す
sub encodeex {
	my $this = shift;
	my $srccode = shift;
	my $destcode = shift;
	my $text = shift;

	if ($srccode ne $destcode) {
		if ($destcode eq 'euc') {
			$text = Jcode->new($text, $srccode)->euc;
		}
		elsif ($destcode eq 'sjis') {
			$text = Jcode->new($text, $srccode)->sjis;
		}
		elsif ($destcode eq 'utf8') {
			$text = Jcode->new($text, $srccode)->utf8;
		}
		elsif ($destcode eq 'jis') {
			$text = Jcode->new($text, $srccode)->jis;
		}
	}
	return $text;
}

# テキストをMIME-Base64形式に変換
sub mimeencode {
	my $this = shift;
	my $srctext = shift;

	# テキストをMIME-Base64形式に変換
	my $desttext = jcode($srctext)->mime_encode;

	return $desttext;
}

# 英数全角英数半角変換(文字コードはシフトJIS)
sub zen2han {
	my $this = shift;
	my $text = shift;
	my $from = '[Ａ-Ｚａ-ｚ０-９，-]';
	my $to = '[A-Za-z0-9]';

	$from = jcode($from, 'utf8')->euc;
	$to = jcode($to, 'utf8')->euc;
	$text = jcode($text, 'sjis')->euc;
	$text = jcode($text)->tr($from, $to)->sjis;

	return $text;
}

# カナの半角を全角に変換(文字コードはシフトJIS)
sub kanah2z {
	my $this = shift;
	my $text = shift;
	$text = Jcode->new($text, 'sjis')->h2z->sjis;
	return $text;
}

1;
# End of String.pm
